# -*- coding: utf-8 -*-

"""Top-level package for UNO Physics Tools."""

__author__ = """Michael Haas"""
__email__ = 'mjhaas@uno.edu'
__version__ = '0.1.0'

from . import config
from . import ladc
from . import nbtools
from . import quantum
from . import wavefuncs

__all__ = [s for s in dir() if not s.startswith('_')]


# -*- coding: utf-8 -*-

import requests

import struct
import datetime
import ftplib
import glob
import sys
import os
import tempfile
from sshtunnel import SSHTunnelForwarder
import pymongo

import numpy as np
try:
    import matplotlib as mpl 
    import matplotlib.pyplot as plt
except:
    print(f'Matplotlib not found')
    mpl = None
    plt = None
from . import wavefuncs
import scipy.signal as signal
import pywt
import pandas as pd

from pathlib import Path
from scipy.io import savemat

from . import config

__all__ = ['EARS', 'getEARSFileUNO', 'getEARSFileUL', 'searchEARS2017',
           'ladcMongoDB', 'get', 'search', 'find', 'Stuff', 'memOpen',
           'create_timeseries', 'create_spec', 'find_interesting', 'MATLAB_format',
           'find_by_datetime']


class ladcMongoDB():
    
    def __init__(self):
        SERVER_HOST = config.server_mongo_uno.address
        SERVER_USER = config.server_mongo_uno.username
        SERVER_PASS = config.server_mongo_uno.password

        self.server = SSHTunnelForwarder(
            SERVER_HOST,
            ssh_username=SERVER_USER,
            ssh_password=SERVER_PASS,
            remote_bind_address=('127.0.0.1', 27017)
        )

        self.server.start()

        self.client = pymongo.MongoClient('127.0.0.1', self.server.local_bind_port) # server.local_bind_port is assigned local port
        self.db = self.client.ladc

    def __enter__(self):
        return self.db
    
    def close(self):
        self.client.close()
        self.server.close()
        
    def __exit__(self, type, value, tb):
        self.close()

class Stuff(object):
    uno_filter = {
                'Buoy': {'$in': ['12', '13', '16', '18', '19', '21']},
                'Disk': '0',
                }

    brydes_calls = {
                    'ETP' : { 
                    'Location':'Eastern Tropical Pacific','Call(s) recorded':['Be1', 'Be2', 'Be3','Be4','Be5','Be6'],
                    'Date':'1999 - 2000'
                    }, 
                    
                    'SCaribbean': {
                    'Location':'Southern Caribbean','Call(s) recorded':['Be7'], 
                    'Date':'February 2000 - March 2000'
                    },
                    
                    'NWPacific': {
                    'Location':'Northwest Pacific','Call(s) recorded':['Be8a', 'Be8b'], 
                    'Date':'Precise date unknown. Recorded "between June and August."'
                    },
                    
                    'GoC' : {
                    'Location':'Gulf of California','Call(s) recorded':['Be10','Be11','Be12,','Be4'],
                    'Date':'1983 - 1986'
                    },
                    
                    'CaboFrio' : {
                    'Location':'Cabo Frio, Brazil','Call(s) recorded':['PSI','LFT','FMT','TM1','TM2'],
                    'Date': 'December 2010 - November 2012'
                    }, 

                    'GoM' : {
                    'Location':'Gulf of Mexico','Call(s) recorded':['Stranded calf', '"Long moans"', '"Downsweep sequences"', '"Tonal sequences"'],
                    'Date': '1988 - 1989 and 2010' 
                    }
                    }  

    # frequency ranges (Be calls) from Rice et al. "Potential Bryde's whale (Balaenoptera edeni) 
    # calls recorded in the northern Gulf of Mexico", add doi numbers
    # except for GoC (Viloria-Gomore et al.)
    frequency_info = { 
                    'ETP' : { 
                    'Be1':(20,23), 'Be2':(35.7,38.2), 'Be3':(2.44,26.9), 'Be4':(59.5,60.2), 'Be5':(26.0,26.8), 'Be6':(57.1,232.7)
                    }, 
                    
                    'SCaribbean': {
                    'Be7':(43.7,48.7)
                    },
                    
                    'NWPacific': {
                    'Be8a':(43.0,48.0), 'Be8b':(137,192)
                    },
                    
                    'GoC' : {
                    'Be10':(79,152), 'Be11':(111,247), 'Be12':(93,145), 'Free-range feeding study':(165,875), 'Calves':(700,900)
                    },
                    
                    'CaboFrio' : {
                    'PSI':(175,674), 'LFT':(7.57,20.39), 'FMT':(336,915), 'TM1':(85.7,123.6), 'TM2':(96,np.nan)
                    },

                    'GoM' : {
                    'Stranded calf':(200,900), '"Long moans"':(43,208), '"Down-sweep sequences"':(51,113), '"Tonal sequences"':(103,103)
                    }
                    } 

def find(skip=0, use_filter=True, **kwargs):
    ''' Find using Mongo object. 
        defaults to use UNO filter for available data in 2017
    '''
    with ladcMongoDB() as db:
        filt = {}
        if use_filter:
            filt = Stuff.uno_filter
        for k in kwargs:
            filt[k] = kwargs[k]
        d = db.detects_2017.find_one(filt, skip=skip)
        return d

def find_by_datetime(datetime_start, datetime_stop=None, buoys=None):
    ''' Find by datetime range, if no stop is given defaults to a 10 minute
        range. buoys takes a list of strings with buoy numbers.
    '''
    if not datetime_stop:
        datetime_stop = datetime_start + datetime.timedelta(minutes=10)
    query = {
        '$and': [{
            'datetime': {
                '$gte': datetime_start
            }
        }, {
            'datetime': {
                '$lte': datetime_stop
            }
        }]
    }
    if buoys:
        buoy_qs = []
        for buoy in buoys:
            buoy_qs.append({'buoy': buoy})
        query['$and'].append({'$or': buoy_qs})
    print(query)
    with ladcMongoDB() as db:
        finds = list(db.fileInfo.find(query))
    return finds


'''
*****
EARS file Classes
*****
'''


class EARS():
    default_fn = r'../data/raw/5234C8C9.021'
    recN = 512
    headerN = 12
    epochs = {'2015': (2000, 1, 1), '2017': (2015, 10, 27)}
    fs = 192000
    fs_time = 32000

    def __init__(self, fn=None, epoch=None, year='2017', norm=False):
        self.data = []
        self.headers = []
        self.timestamps = []
        if fn is None:
            self.filename = self.default_fn
        else:
            self.filename = fn
        if year is not None:
            self.year = year
        else:
            if os.path.basename(self.filename)[0]=='7':
                self.year = '2017'
            else:
                self.year = '2015'
        if epoch is not None:
            self.epoch = epoch
        else:
            self.epoch = self.epochs.get(self.year, self.epochs['2015'])
        self.norm = norm
        self.load(self.filename)

    def load(self, fn=None, norm=None):
        if not norm:
            norm = self.norm
        if fn is not None:
            self.filename = fn
        with open(self.filename, 'rb') as f:
            d = f.read()
        data = []
        headers = []
        timestamps = []
        dataN = int(len(d) / self.recN)
        for i in range(dataN):
            data.extend(
                struct.unpack_from(
                    '>250h', d, offset=self.headerN + self.recN * i))
            if i == 0 or headers[-1] != d[0:self.headerN]:
                h = d[0:self.headerN]
                headers.append(h)
                timestamps.append(self.header2timestamp(h))
        self.time_0 = timestamps[0]
        self.time_f = self.time_0 + datetime.timedelta(
            seconds=len(data) / self.fs)
        self.timestamps = timestamps
        self.headers = headers
        if norm:
            data -= np.mean(data)
            data /= np.max(np.abs(data))
        self.data = data

    def header2timestamp(self, h):
        s = struct.unpack('6x6B', h)
        timestampSeconds = ((
            (s[0] - 14) / 16) * 2**40 + s[1] * 2**32 + s[2] * 2**24 +
                            s[3] * 2**16 + s[4] * 2**8 + s[5] * 2**0) / self.fs_time
        return (datetime.timedelta(seconds=timestampSeconds) +
                datetime.datetime(*self.epoch))

def getEARSFileUNO(fn=None, outDir='', warnings=False, directory=None):
    if not fn:
        return None
    try:
        if not directory:
            Buoy = int(fn[-3:][:2])
            Disk = int(fn[-3:][2:])
            directory = 'Buoy{:0>2d}{:0>1d}'.format(Buoy, Disk)
        ftp = ftplib.FTP(config.server_ftp_uno.address)
        ftp.login(user=config.server_ftp_uno.username, passwd=config.server_ftp_uno.password)
        
        ftp.cwd(directory)
        #print(directory)
        fn_out = os.path.join(outDir, fn)
        response = ftp.retrbinary('RETR {}'.format(fn),
                                  open(fn_out, 'wb').write)
        #print(response)
        ftp.close()
        return fn_out
    except Exception as e:
        if warnings:
            print(f'Problem getting {fn} Error: {str(e)}')
        return None


def getEARSFileUL(fn=None, outDir='', warnings=False):
    if not fn:
        return None
    try:
        Buoy = int(fn[-3:][:2])
        Disk = int(fn[-3:][2:])
        ftp = ftplib.FTP(config.server_ftp_ull.address)
        ftp.login(user=config.server_ftp_ull.username, passwd=config.server_ftp_ull.password)
        if Buoy < 6:
            directory = '/Volumes/FirstRAID/'
        elif Buoy == 6 and Disk == 1:
            directory = '/Volumes/FirstRAID/'
        elif Buoy == 6 and Disk == 0:
            directory = '/Volumes/FirstRAID/'
        else:
            directory = '/Volumes/SecondRAID/'
        directory += 'Buoy{:0>2d}_DISK{:0>1d}'.format(Buoy, Disk)
        ftp.cwd(directory)
        #print(directory)
        fn_out = os.path.join(outDir, fn)
        response = ftp.retrbinary('RETR {}'.format(fn),
                                  open(fn_out, 'wb').write)
        #print(response)
        ftp.close()
        return fn_out
    except Exception as e:
        if warnings:
            print(f'Problem getting {fn} Error: {str(e)}')
        return None

def searchEARS2017(searchData={}):
    searchJson = {k: str(v) for k,v in searchData.items()}
    r = requests.post(config.server_api_uno.address, json=searchJson)
    return r.json()

def search(searchData={}, year='2017'):
    ''' Search using requests api. '''
    if year=='2015':
        return searchEARS2015(searchData=searchData)
    if year=='2017':
        return searchEARS2017(searchData=searchData)
        

def get(fn=None, outDir='', warnings=True, directory=None):
    '''Gets a file, returns filename, else returns None'''
    if directory:
        return getEARSFileUNO(fn=fn, outDir=outDir, warnings=warnings, directory=directory)
    if not fn:
        fn = '5176009D.010'
    try:
        if fn[0]=='7':
            return getEARSFileUNO(fn=fn, outDir=outDir, warnings=warnings)
        elif fn[0]=='5':
            return getEARSFileUL(fn=fn, outDir=outDir, warnings=warnings)
        else:
            return None
    except Exception as e:
        if warnings:
            print(f'Problem getting {fn} Error: {str(e)}')
        return None

def memOpen(fn, warnings=True, directory=None):
    '''Gets file from network and returns EARS object, does not write to storage.'''
    with tempfile.TemporaryDirectory() as tmpdirname:
        fni = get(fn, outDir=tmpdirname, warnings=warnings, directory=directory)
        return EARS(fni)

def create_timeseries(filename=None, skip=None, show_plt=True):
    if filename is None:
        if skip is None:
            detect = find()
        elif skip is not None:
            detect = find(skip=skip)
        if detect is None:
            return False
        filename = detect['filename']
        b = EARS(fn=get(filename))
        timestamp = b.time_0
        plt.plot(b.data)
        plt.title(f'File {filename} - Recorded {timestamp}')
        if show_plt == True:
            plt.show()

def create_spec(skip=None, cmap='nipy_spectral', figsize=(6,4), save_fig=None, show_plt=True, filename=None, downloadname=None): 
    if filename is None:
        if skip is None:
            detect = find()
        elif skip is not None:
            detect = find(skip=skip)
        if detect is None:
            return False
        filename = detect['filename']
        b = EARS(fn=get(filename))
    elif filename is not None:
        b = EARS(fn=get(filename))
    timestamp = b.time_0
    fig, axes = plt.subplots(1,1,figsize=figsize)
    f, t, Sxx = signal.spectrogram(wavefuncs.wave_clean(b.data), b.fs, window='hann')
    axes.set_ylabel('Frequency [Hz]')
    axes.set_xlabel('Time [sec]')
    plt.tight_layout()
    plt.title(f'File {filename} - Recorded {timestamp}')
    axes.pcolormesh(t, f, 10*np.log10(Sxx), cmap=cmap)
    if show_plt == True:
        fig.show()
    if save_fig is not None:
        naming = downloadname
        getdir = os.getcwd()
        newfilename = (f'{getdir}\\{naming}')
        plt.savefig(newfilename)
        return filename
    else:
        if save_fig is None:
            return None

def find_interesting(skip_start=0, number_of_files=9, Type=6, Buoy='13', Disk='0'): 
    # change skip_start number to get a new set
    # keep number of files square to make plotting below work smoothly
    records = []
    with ladcMongoDB() as db:
        to_find = {
                'type': Type,  # type 6 is the highest frequency band ULL looked for
                'Buoy': Buoy,  # 13 is one of the buoys we have here at UNO and...
                'Disk': Disk  # disk 0 is one we have locally
                }
        cursor = db.detects_2017.find(to_find, skip=skip_start)
        '''found = cursor.count()
        if found < number_of_files:
            number_of_files = found'''
        for i in range(number_of_files):
            records.append(cursor.next())
    return records

def MATLAB_format(plot=True, show_plt=False, save_plt=True, clip_length=577, number_of_files=9, skip_start=0, directory='data', records=None, Type=6, Buoy='13', Disk='0'):
    save_folder = Path(directory)
    save_folder.mkdir(exist_ok=True)
    if plot:
        size_of_plots = int(round(np.sqrt(number_of_files)))
        fig, ax = plt.subplots(size_of_plots, size_of_plots, figsize=(16, 16))
        axF = ax.reshape(-1)
    clips = []
    if records is None:
        records = find_interesting(number_of_files=number_of_files, skip_start=skip_start, Type=Type, Buoy=Buoy, Disk=Disk)
    for i, record in enumerate(records):
        e = memOpen(record['filename'])
        start_n = record['startRecord'] * 250
        clipped = e.data[start_n:start_n + clip_length]
        savemat(str(save_folder.joinpath(f'click{i}.mat')), {'data': clipped, **record})
        clips.append(clipped)
        if plot:
            axF[i].plot(clipped)
    if plot:
        if save_plt:
            plt.savefig(str(save_folder.joinpath('clips.jpg')))
        if show_plt:
            plt.show()
        else:
            plt.close()
    clips_dictionary = {f'click{i}_data': c for i, c in enumerate(clips)}
    clips_dictionary['records'] = records
    savemat(str(save_folder.joinpath(f'ALL_clicks.mat')), clips_dictionary)
    savemat(str(save_folder.joinpath(f'ALL_clicksMAT.mat')), {'clips': clips})



import sympy
from IPython.display import Latex, Markdown, display

__all__ = ['pmath', 'nbprint', 'setup_notebook']


def nbprint(string):
    display(Markdown(string))


lt = sympy.latex
sympy.init_printing()


def pmath(sym, ret=False, pre='', post='', preM='', postM=''):
    output = f'{pre}${preM}{lt(sym)}{postM}${post}'
    if ret:
        return output
    nbprint(output)


def setup_notebook():
    #global sp, np, pmath, nbprint, plt
    global sympy
    print(sympy.I)
    #from unophysics.nbtools import pmath, nbprint
    #import sympy
    #import numpy as np
    #import matplotlib.pyplot as plt
    print(dir())

# def reset_notebook_test1():
#    global sp, np, pmath, nbprint, norm, plt, ureg
#     %reset_selective -f "^((?!reset_notebook).)*$"
#     import pint
#     ureg = pint.UnitRegistry()
#     from unophysics.nbtools import pmath, nbprint
#     import sympy as sp
#     import numpy as np
#     from scipy.stats import norm
#     %matplotlib inline
#     import matplotlib.pyplot as plt



import matplotlib as mpl
import numpy as np
import os
import pandas as pd
import matplotlib.pyplot as plt
import tkinter 

import datetime
import struct
import ftplib
import glob
import tempfile
import pymongo
import requests
import sys

import scipy.signal as signal
import scipy.io.wavfile
import pywt

from unophysics import ladc, wavefuncs
from pathlib import Path
from scipy.io import savemat
from sshtunnel import SSHTunnelForwarder

root = tkinter.Tk()
root.title('LADC-GEMM Interactive')
root.geometry('540x260')
root.grid_columnconfigure(1, minsize=135)

# SET UP INFORMATION PAGE FOR EACH LOCATION
localcalls = []
def info_location_change():
    currentlocation.set(ladc.Stuff.brydes_calls[info_location.get()]['Location'])
    locationlabel.grid(column=1, row=0, sticky=tkinter.N+tkinter.W)

    global localcalls
    localcalls = ladc.Stuff.brydes_calls[info_location.get()]['Call(s) recorded']
    numcallsstring = ', '.join(localcalls)
    current_numcalls.set(numcallsstring)
    numcalls_label.grid(column=1, row=0, sticky=tkinter.N+tkinter.W, pady=5)
    callmenu.delete(0, tkinter.END)
    for x in localcalls:
        callmenu.insert(tkinter.END, x)
    current_date.set(ladc.Stuff.brydes_calls[info_location.get()]['Date'])
    date_label.grid(column=1, row=0, sticky=tkinter.N+tkinter.W, pady=5)
    current_minmax.set('')

# LOCATION BUTTONS
infobutton_box = tkinter.LabelFrame(root)
info_location = tkinter.StringVar()
info_location.set('ETP')

currentlocation=tkinter.StringVar()
locationlabel = tkinter.Label(root, textvariable=currentlocation, font=('bold', 13))

etp = tkinter.Radiobutton(infobutton_box, text='Eastern Tropical Pacific', variable=info_location, value='ETP', command=info_location_change)
scarib = tkinter.Radiobutton(infobutton_box, text='Southern Caribbean', variable=info_location, value='SCaribbean', command=info_location_change)
nwpac = tkinter.Radiobutton(infobutton_box, text='Northwest Pacific', variable=info_location, value='NWPacific', command=info_location_change)
goc = tkinter.Radiobutton(infobutton_box, text='Gulf of California', variable=info_location, value='GoC', command=info_location_change)
cabo = tkinter.Radiobutton(infobutton_box, text='Cabo Frio, Brazil', variable=info_location, value='CaboFrio', command=info_location_change)
gom = tkinter.Radiobutton(infobutton_box, text='Gulf of Mexico', variable=info_location, value='GoM', command=info_location_change)

# REGIONAL CALL INFORMATION WIDGETS
numcallsbox = tkinter.LabelFrame(root)
numcalls = tkinter.Label(numcallsbox, text='Call(s) recorded: ')
current_numcalls = tkinter.StringVar()
numcalls_label = tkinter.Label(numcallsbox, textvariable=current_numcalls, wraplength=200, justify='left')

datebox = tkinter.LabelFrame(root)
date = tkinter.Label(datebox, text='Date: ')
current_date = tkinter.StringVar()
date_label = tkinter.Label(datebox, textvariable=current_date, wraplength=200, justify='left')

# FREQUENCY INFORMATION WIDGETS
freqbox = tkinter.LabelFrame(root)
freq = tkinter.Label(freqbox, text='Frequency information: ')

def change_facts(event):
    selecttuple = event.widget.curselection()
    selectindex = selecttuple[0]
    callname = localcalls[selectindex]
    minmaxrange = ladc.Stuff.frequency_info[info_location.get()][callname]
    minbookmark = minmaxrange[0]
    maxbookmark = minmaxrange[1]
    minmax_string = f'{minbookmark} - {maxbookmark} Hz'
    current_minmax.set(minmax_string) 

callbox = tkinter.Frame(freqbox)
call = tkinter.Label(callbox, text='Call: ')
callmenu = tkinter.Listbox(callbox, selectmode='SINGLE', height=6) 
callmenu.bind('<<ListboxSelect>>', change_facts)

minmaxbox = tkinter.Frame(freqbox)
minmax = tkinter.Label(minmaxbox, text='Min/max frequencies: ')
current_minmax = tkinter.StringVar()
minmax_label = tkinter.Label(minmaxbox, textvariable=current_minmax) 

# MENU WIDGET CHANGES
def datapage():
    reset_inputs()

    root.grid_columnconfigure(index=1, minsize=135)
    root.grid_rowconfigure(index=0, minsize=10)
    root.grid_rowconfigure(index=2, minsize=10)
    
    widgetlist = [minmax_label, callbox, minmaxbox, freqbox, freq, call, callmenu, minmax, infobutton_box, etp, scarib, nwpac, goc, cabo, gom, datebox, date, numcallsbox, numcalls, locationlabel, numcalls_label, date_label]
    for widget in widgetlist:
        widget.grid_remove()

    onefilename_box.grid(column=1, row=0, sticky=tkinter.W+tkinter.E, padx=10, pady=10)
    enter_onefile.grid(column=1, row=0, pady=5, padx=3)

    skipval_box.grid(column=2, row=0, sticky=tkinter.W+tkinter.E, padx=10, pady=10)
    enter_skipval.grid(column=2, row=0, pady=5, padx=3)

    cmap_box.grid(column=1, row=1, padx=10, pady=10, sticky=tkinter.E+tkinter.W)
    colorschemes.grid(column=1, row=1, sticky=tkinter.E+tkinter.W)

    fileamount_box.grid(column=2, row=1, padx=10, pady=10, sticky=tkinter.E+tkinter.W)
    fileamounts.grid(column=2, row=1, sticky=tkinter.E+tkinter.W)

    reset.grid(column=1, row=2, sticky=tkinter.E+tkinter.W, pady=10, padx=10)

    show_button.grid(column=2, row=2, sticky=tkinter.E+tkinter.W, padx=10, pady=10)

    databutton_box.grid(column=0, row=0, rowspan=4, padx=10, pady=10, sticky=tkinter.N+tkinter.S)
    spec_plot.grid(row=0, sticky=tkinter.W, pady=15, padx=5)
    amp_plot.grid(row=1, sticky=tkinter.W, pady=15, padx=5)
    search_interesting.grid(row=2, sticky=tkinter.W, pady=15, padx=5)

def infopage():
    info_location.set(None)

    root.grid_columnconfigure(index=1, minsize=330)
    root.grid_rowconfigure(index=0, minsize=30)
    root.grid_rowconfigure(index=2, minsize=20)

    widgetlist = [onefilename_box, enter_onefile, skipval_box, enter_skipval, cmap_box, colorschemes, fileamount_box, fileamounts, reset, show_button, databutton_box, spec_plot, amp_plot, search_interesting]
    for widget in widgetlist:
        widget.grid_remove()

    infobutton_box.grid(column=0, row=0, padx=10, pady=10, rowspan=6, sticky=tkinter.N+tkinter.S)

    etp.grid(column=0, row=0, sticky=tkinter.W, padx=10, pady=5)
    scarib.grid(column=0, row=1, sticky=tkinter.W, padx=10, pady=5)
    nwpac.grid(column=0, row=2, sticky=tkinter.W, padx=10, pady=5)
    goc.grid(column=0, row=3, sticky=tkinter.W, padx=10, pady=5)
    cabo.grid(column=0, row=4, sticky=tkinter.W, padx=10, pady=5)
    gom.grid(column=0, row=5, sticky=tkinter.W, padx=10, pady=5)

    numcallsbox.grid(column=1, row=1, columnspan=2, sticky=tkinter.W+tkinter.E+tkinter.N+tkinter.S)
    numcalls.grid(column=0, row=0, sticky=tkinter.W+tkinter.N, padx=10, pady=5)
    datebox.grid(column=1, row=2, columnspan=2, sticky=tkinter.W+tkinter.E+tkinter.N+tkinter.S)
    date.grid(column=0, row=0, sticky=tkinter.W+tkinter.N, padx=10, pady=5)

    freqbox.grid(column=1, row=3, columnspan=2, sticky=tkinter.W+tkinter.E+tkinter.N+tkinter.S)
    freq.grid(column=0, row=0)

    callbox.grid(column=0, row=1)
    call.grid(column=0, row=0)
    callmenu.grid(column=1, row=0)

    minmaxbox.grid(column=1, row=1)
    minmax.grid(column=0, row=0)
    minmax_label.grid(column=1, row=0)
    callmenu.delete(0, tkinter.END)
    current_minmax.set('')

# CREATING MENU
menu = tkinter.Menu(root)
root.config(menu=menu)
datamenu = tkinter.Menu(menu)
menu.add_cascade(label='Data', menu=datamenu)
datamenu.add_command(label='Data', command=datapage)
infomenu = tkinter.Menu(menu)
menu.add_cascade(label='Information', menu=infomenu)
infomenu.add_command(label='Information', command=infopage)

# TYPE IN FILE NAME
onefilename_box = tkinter.LabelFrame(root, text='Filename')
onefilename_box.grid(column=1, row=0, sticky=tkinter.W+tkinter.E, padx=10, pady=10)
onefilename = tkinter.StringVar()
onefilename.set('')
enter_onefile = tkinter.Entry(onefilename_box, textvariable=onefilename, width=16)
enter_onefile.grid(column=1, row=0, pady=5, padx=3)

# TYPE IN SKIP VALUE
def skip_error_check(number):
    valid = False
    if number.isdigit():
        if (int(number) <= 10000) and (int(number) >= 0):
            valid = True
    elif number == '':
        valid = True
    return valid
validate_skip = (root.register(skip_error_check), '%P')
skipval_box = tkinter.LabelFrame(root, text='Skip Value')
skipval_box.grid(column=2, row=0, sticky=tkinter.W+tkinter.E, padx=10, pady=10)
skipval = tkinter.StringVar()
skipval.set(0)
enter_skipval = tkinter.Spinbox(skipval_box, from_=0, to=10000, textvariable=skipval, width=16, validate='all', validatecommand=validate_skip)
enter_skipval.grid(column=2, row=0, pady=5, padx=3)

# OPTION MENUS
def show_options():
    global cmap_box, colorschemes, fileamount_box, fileamounts, fileamount_str, enter_onefile
    if current_page.get() == 'spec':
        enter_onefile.configure(state='normal')
        fileamounts.configure(state='disabled')
        colorschemes.configure(state='normal')
        colorscheme_str.set(cmaps[2])
        skipval.set(0)
        fileamount_str.set(number_of_files[0])
        onefilename.set('')
    else: 
        colorschemes.configure(state='disabled')
    
    if current_page.get() == 'amp':
        enter_onefile.configure(state='normal')
        fileamounts.configure(state='disabled')
        colorschemes.configure(state='disabled')
        skipval.set(0)
        fileamount_str.set(number_of_files[0])
        onefilename.set('')
    
    if current_page.get() == 'interesting':
        enter_onefile.configure(state='disabled')
        fileamounts.configure(state='normal')
        colorschemes.configure(state='disabled')
        skipval.set(0)
        fileamount_str.set(number_of_files[0])
    else:
        fileamounts.configure(state='disabled')

    if current_page.get() != 'spec' and current_page.get() != 'amp' and current_page.get() != 'interesting':
        enter_onefile.configure(state='disabled')
        fileamounts.configure(state='disabled')
        colorschemes.configure(state='disabled')
        enter_skipval.configure(state='disabled')
        reset.configure(state='disabled')
        show_button.configure(state='disabled')

cmaps = ['hsv', 'Greys', 'nipy_spectral']
cmap_box = tkinter.LabelFrame(root, text='Color Scheme')
colorscheme_str = tkinter.StringVar()
colorscheme_str.set(cmaps[2])
colorschemes = tkinter.OptionMenu(cmap_box, colorscheme_str, *cmaps)
cmap_box.grid(column=1, row=1, padx=10, pady=10, sticky=tkinter.E+tkinter.W)
colorschemes.grid(column=1, row=1, sticky=tkinter.E+tkinter.W)

number_of_files = ['4','9','16']
fileamount_box = tkinter.LabelFrame(root, text='Number of Files')
fileamount_str = tkinter.StringVar()
fileamount_str.set(number_of_files[0])
fileamounts = tkinter.OptionMenu(fileamount_box, fileamount_str, *number_of_files)
fileamount_box.grid(column=2, row=1, padx=10, pady=10, sticky=tkinter.E+tkinter.W)
fileamounts.grid(column=2, row=1, sticky=tkinter.E+tkinter.W)

# PLOT TYPE RADIOBUTTONS
# RADIOBUTTON LABELFRAME
databutton_box = tkinter.LabelFrame(root)
databutton_box.grid(column=0, row=0, rowspan=4, padx=10, pady=10, sticky=tkinter.N+tkinter.S)

current_page = tkinter.StringVar()
current_page.set(None)
spec_plot = tkinter.Radiobutton(databutton_box, text='Create Spectrogram', command=show_options, variable=current_page, value='spec')
spec_plot.grid(row=0, sticky=tkinter.W, pady=15, padx=5)

amp_plot = tkinter.Radiobutton(databutton_box, text='Create Time Series', command=show_options, variable=current_page, value='amp')
amp_plot.grid(row=1, sticky=tkinter.W, pady=15, padx=5)

search_interesting = tkinter.Radiobutton(databutton_box, text= 'Find Interesting', command=show_options, variable=current_page, value='interesting')
search_interesting.grid(row=2, sticky=tkinter.W, pady=15, padx=5)

# RESET BUTTON
def reset_inputs():
    global onefilename, skipval, current_page, cmap_box, colorschemes, fileamount_box, fileamounts, fileamount_str, colorscheme_str
    onefilename.set('')
    skipval.set(0)
    current_page.set(None)
    fileamount_str.set(number_of_files[0])
    colorscheme_str.set(cmaps[2])
    enter_onefile.configure(state='normal')
    fileamounts.configure(state='normal')
    colorschemes.configure(state='normal')

reset = tkinter.Button(root, text='Reset All', command=reset_inputs)
reset.grid(column=1, row=2, sticky=tkinter.E+tkinter.W, pady=10, padx=10)

# FORMAT USER INPUT FOR _ladc FUNCTIONS
trueskip = 0
truefilename = ''
fileamount_int = 0

def recordswindow(recordfn=None, recordskip=None, recordnumber=None):
    global trueskip, truefilename, fileamount_int

    recordswindow = tkinter.Toplevel(root)
    recordswindow.title('Records')
    recordswindow.geometry('630x300')

    truerecords = tkinter.StringVar()
    truerecords.set('')

    trueheader = tkinter.StringVar()
    trueheader.set('')

    recordlabel = tkinter.Label(recordswindow, textvariable=truerecords, wraplength=600, font=('bold', 12), justify='left')
    headerlabel = tkinter.Label(recordswindow, textvariable=trueheader, font=(6))
    
    if current_page.get() == 'spec' or current_page.get() == 'amp':
        if recordfn is None:
            if recordskip is None:
                detect = ladc.find()
            elif recordskip is not None:
                detect = ladc.find(skip=recordskip)
        elif recordfn is not None:
            detect = ladc.find(filename=recordfn)

        detectbookmark = str(detect)
        truerecords.set(detectbookmark)
        
        filenamebookmark = detect['filename']
        headerbookmark = (f'File {filenamebookmark} Records')
        trueheader.set(headerbookmark)

    if current_page.get() == 'interesting':
        if recordnumber is not None:
            truerecords.set(ladc.find_interesting(skip_start=recordskip, number_of_files=recordnumber, Type=6, Buoy='13', Disk='0'))
            trueheader.set('File Records')

    recordlabel.grid(column=0, row=1, sticky=tkinter.E+tkinter.W, padx=5, pady=5)
    headerlabel.grid(column=0, row=0, sticky=tkinter.N+tkinter.W, padx=5, pady=5)
    print('window')

# POPUP WINDOW
def build_plot(): 
    global trueskip, truefilename, fileamount_int
    print('built plot')

    truefilename = onefilename.get()
    if truefilename == '':
        truefilename = None
    
    fileamount_int = int(fileamount_str.get())

    trueskip = int(skipval.get())
    if trueskip == 0:
        trueskip = None

    if current_page.get() == 'spec' or current_page.get() == 'amp':
        if (trueskip is not None) and (truefilename is not None):
            errorwindow = tkinter.Toplevel(root)
            errorwindow.title('Error')
            errormessage = tkinter.Label(errorwindow, text='Please enter only a filename OR a skip value.')
            errormessage.grid(column=0, row=0, sticky=tkinter.E+tkinter.W, padx=10, pady=10)
            okbutton = tkinter.Button(errorwindow, text='Okay', command=errorwindow.destroy)
            okbutton.grid(column=0, row=1, sticky=tkinter.E+tkinter.W, padx=10, pady=10)
        else:
            try:
                recordswindow(recordfn=truefilename, recordskip=trueskip, recordnumber=None)
                if current_page.get() == 'spec': # use the output from the window building function instead of getting it from the widget
                    ladc.create_spec(skip=trueskip, cmap=(colorscheme_str.get()), figsize=(6,4), save_fig=None, show_plt=True, filename=truefilename)
                if current_page.get() == 'amp':
                    ladc.create_timeseries(filename=truefilename, skip=trueskip, show_plt=True)
            except FileNotFoundError:
                errorwindow = tkinter.Toplevel(root)
                errorwindow.title('Error')
                errormessage = tkinter.Label(errorwindow,text='Sorry, that file does not exist.')
                errormessage.grid(column=0, row=0, sticky=tkinter.E+tkinter.W, padx=10, pady=10)
                okbutton = tkinter.Button(errorwindow, text='Okay', command=errorwindow.destroy)
                okbutton.grid(column=0, row=1, sticky=tkinter.E+tkinter.W, padx=10, pady=10)

    if current_page.get() == 'interesting':
        recordswindow(recordfn=None, recordskip=trueskip, recordnumber=fileamount_int)
        ladc.find_interesting(skip_start=trueskip, number_of_files=fileamount_int, Type=6, Buoy='13', Disk='0')
        ladc.MATLAB_format(plot=True, show_plt=True, save_plt=False, clip_length=577, number_of_files=fileamount_int, directory='data', records=None, Type=6, Buoy='13', Disk='0', skip_start=trueskip)

show_button = tkinter.Button(root, text='Create Figure(s)', command=build_plot)
show_button.grid(column=2, row=2, sticky=tkinter.E+tkinter.W, padx=10, pady=10)

root.mainloop()


from pathlib import Path
import json
import collections

Server = collections.namedtuple('Server',
    ['address', 'username', 'password'])

__all__ = ['server_mongo_uno', 'server_ftp_uno', 'server_ftp_ull', 'server_api_uno', 'load_credentials']

def load_credentials(cred_fn='unophysics.cred'):
    global server_mongo_uno, server_ftp_uno, server_ftp_ull, server_api_uno
    creds = {}

    try:
        fn = next(Path.home().glob(cred_fn))
        print(fn)
        with open(fn) as f:
            creds = json.load(f)
    except StopIteration:
        print(f'WARNING --- {cred_fn} file not found')
    except Exception as e:
        print(f'WARNING --- {cred_fn} file not available, error:\n\t{e.__repr__()}')

    server_mongo_uno = Server(
        address = creds.get('server_mongo_uno', {}).get('address', ''),
        username = creds.get('server_mongo_uno', {}).get('username', ''),
        password = creds.get('server_mongo_uno', {}).get('password', ''),
        )

    server_ftp_uno = Server(
        address = creds.get('server_ftp_uno', {}).get('address', ''),
        username = creds.get('server_ftp_uno', {}).get('username', ''),
        password = creds.get('server_ftp_uno', {}).get('password', ''),
        )

    server_ftp_ull = Server(
        address = creds.get('server_ftp_ull', {}).get('address', ''),
        username = creds.get('server_ftp_ull', {}).get('username', ''),
        password = creds.get('server_ftp_ull', {}).get('password', ''),
        )

    server_api_uno = Server(
        address = creds.get('server_api_uno', {}).get('address', ''),
        username = creds.get('server_api_uno', {}).get('username', ''),
        password = creds.get('server_api_uno', {}).get('password', ''),
        )

load_credentials()


import numpy as np
import pywt

__all__ = ['threshold', 'thresh_wave_coeffs', 'wave_clean']

default_wavelet = 'db20'
default_thresh = 50

def threshold(x_in, delta=default_thresh, hard=False):
    x_thresh_indicies = np.abs(x_in) < delta
    if not hard:     
        x_in = np.sign(x_in)*(np.abs(x_in) - delta)
    x_in[x_thresh_indicies] = 0
    return x_in

def thresh_wave_coeffs(d_w, delta=default_thresh, hard=False):
    d_w_out = [np.asarray(threshold(d, delta = delta, hard=hard)) for d in d_w]
    return d_w_out


def wave_clean(data, wavelet=None, wavelet_thresh=None, ret_thresh=False, hard=False):
    if wavelet is None:
        wavelet = default_wavelet
    data = np.asarray(data)
    data = data - data.mean()
    d_w = pywt.wavedec(data, wavelet)
    if wavelet_thresh is None:
        #coeffs = pywt.dwtn(e.data, wavelet='db2')
        detail_coeffs = d_w[-1]
        denom = 0.6744897501960817
        sigma = np.median(np.abs(detail_coeffs)) / denom
        wavelet_thresh = sigma*np.sqrt(2*np.log(len(data)))
    d_w_t = thresh_wave_coeffs(d_w, delta=wavelet_thresh, hard=hard)
    clean = pywt.waverec(d_w_t, wavelet)
    if ret_thresh:
        return clean, wavelet_thresh
    return clean


import sympy as sp

__all__ = ["H", "E", "P", "phi"]


def H(psi_in, v_in, mass, var):
    hbar= sp.symbols('hbar', real=True)
    return -(1 *hbar**2) / (2 * mass) * sp.diff(psi_in, var, 2) + v_in * psi_in

def E(phi_in, v_in, mass, var, low, high):
    return sp.integrate(phi_in * H(phi_in, v_in, mass, var),
                           (var, low, high))

def P(phi_in, psi_in, var, low, high):
    return (sp.Abs(sp.integrate(phi_in * psi_in, (var, low, high)))**2)/sp.integrate(psi_in * psi_in, (var, low, high))

def phi(n, a=1, x=None):
    if not x:
        x = sp.symbols('x')
    return (sp.sqrt(2)/sp.sqrt(a))*sp.sin((n * sp.pi * x) / a)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md') as history_file:
    history = history_file.read()

requirements = ['Click>=6.0', 'requests', 'sshtunnel', 'pymongo', 'sympy', 'jupyter', 'numpy', 'matplotlib', 'scipy', 'PyWavelets', 'pandas']

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="Michael Haas",
    author_email='mjhaas@uno.edu',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Tools for physics from UNO.",
    entry_points={
        'console_scripts': [
            'unophysics=unophysics.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description_content_type='text/markdown',
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='unophysics',
    name='unophysics',
    packages=find_packages(include=['unophysics']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/micha2718l/unophysics',
    version='0.4.7',
    zip_safe=False,
)
